﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
    class Level
    {
        public Vector2 LevelDimensions{ get;  set;}
        private List<Entity> entities;

        private Level()
        {
            entities = new List<Entity>();
        }
        //private Level(List<Entity> entities)
        //{
        //    this.entities = entities;   // reference contained inside the object
        //}
        public void LoadContent()
        {
            throw new NotImplementedException();
        }
        public void Update(GameTime gameTime)
        {
        }
        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            foreach (Entity entity in entities)
            {
                entity.Draw(spriteBatch);
            }
        }
        public static Level LoadSvg(string filename)
        {
            Level result = new Level();

            //READER SETTINGS
            XmlReaderSettings settings = new XmlReaderSettings();
            settings.IgnoreComments = true;
            settings.IgnoreWhitespace = true;

            //PARSE
            using (XmlReader r = XmlReader.Create(filename, settings))
            {
                while (r.Read())
                {
                    string id = "";
                    
                    //TODO: Read level size from file
                    result.LevelDimensions = new Vector2(1000, 1000); 

                    #region PATH
                    if (r.Name == "path")
                    {
                        // Parse style attribute
                        string style = "";
                        if (r.MoveToAttribute("style"))
                            style = r.ReadContentAsString();

                        // stroke color
                        string strokeColor="";
                        if (r.MoveToAttribute("stroke"))
                            strokeColor = r.ReadContentAsString();
                        else
                            strokeColor = GetAttributeFromStyle("stroke", style);
                        
                        //OPTICAL ENTITY
                        if (strokeColor != "none")  //if the path has a stroke color it is an optical entity
                        {
                            // d
                            string d = "";
                            if (r.MoveToAttribute("d"))
                                d = r.ReadContentAsString();
                            // id
                            if (r.MoveToAttribute("id"))
                                id = r.ReadContentAsString();
                            
                            // Create entities
                            List<Entity> pathEntities = CreateEntitiesFromPath(d, strokeColor, id);
                            if (pathEntities.Count > 0)
                                result.Entities.AddRange(pathEntities);
                        }
                        
                        //ATTRIBUTE FILL //TODO?
                        //  CreateFillMap
                    }
                    #endregion

                    //RECT //TODO
                    //  CheckId
                    //  CalculateCenter
                    //  SetSpawnpoint
                }
            }

            return result;
        }
        /// <summary>
        /// Parse the style attribute of an SVG element
        /// </summary>
        /// <param name="attribute">The attribute name to look for</param>
        /// <param name="style">A string containing the style attribute</param>
        /// <returns>A string with the attribute value. "none" if the attribute does not exist</returns>
        private static string GetAttributeFromStyle(string attribute, string style)
        {
            attribute += ":";
            int startIndex = style.IndexOf(attribute);
            if (startIndex < 0) //the attribute could not be found
                return "none";
            startIndex += attribute.Length; //change startIndex to where the value starts
            int endIndex = style.IndexOf(';', startIndex);
            return style.Substring(startIndex, endIndex - startIndex);
        }
        private static List<Entity> CreateEntitiesFromPath(string d, string strokeColor, string id)
        {
            List<Entity> result = new List<Entity>();
            List<IShape> shapes = ParseD(d);

            if (strokeColor == "#000000")   //wall
            {
                foreach (IShape shape in shapes)
                {
                    result.Add(new Wall(shape));
                }
            }
            else if (strokeColor == "#ff0000")  //mirror
            {
                foreach (IShape shape in shapes)
                {
                    result.Add(new Mirror(shape));
                }
            }
            return result;
        }   
        /// <summary>
        /// Convert each part of the path into matching shape objects
        /// </summary>
        /// <param name="d">Must contain only absolute coordinates</param>
        /// <returns>A list of the shapes generated from the path</returns>
        private static List<IShape> ParseD(string d)
        {
            List<IShape> shapes = new List<IShape>();
            string[] elements = d.Split(' ', ',');

            //SETTINGS
            System.Globalization.CultureInfo ci = (System.Globalization.CultureInfo)System.Globalization.CultureInfo.CurrentCulture.Clone();
            ci.NumberFormat.NumberDecimalSeparator = ".";

            Vector2 lastCoordinate = Vector2.Zero;
            Vector2 firstCoordinate = Vector2.Zero;
            for(int i=0; i<elements.Length;)
            {
                if (elements[i] == "M")
                {
                    firstCoordinate.X = float.Parse(elements[i + 1], ci);
                    firstCoordinate.Y = float.Parse(elements[i + 2], ci);
                    lastCoordinate = firstCoordinate;
                    i += 2;
                }
                else if (elements[i] == "L")
                {
                    Vector2 destination;
                    destination.X = float.Parse(elements[i + 1], ci);
                    destination.Y = float.Parse(elements[i + 2], ci);
                    shapes.Add(new LineShape(lastCoordinate, destination));
                    lastCoordinate = destination;
                    i += 2;
                }
                else if (elements[i] == "C")
                {
                    Vector2 cord1, cord2, cord3;
                    cord1.X = float.Parse(elements[i + 1], ci);
                    cord1.Y = float.Parse(elements[i + 2], ci);
                    cord2.X = float.Parse(elements[i + 3], ci);
                    cord2.Y = float.Parse(elements[i + 4], ci);
                    cord3.X = float.Parse(elements[i + 5], ci);
                    cord3.Y = float.Parse(elements[i + 6], ci);
                    shapes.Add(new BezierShape(lastCoordinate, cord1, cord2, cord3));
                    lastCoordinate = cord3;
                    i += 6;
                }
                else if (elements[i] == "H")
                {
                    Vector2 destination;
                    destination.X = float.Parse(elements[i + 1], ci);
                    destination.Y = lastCoordinate.Y;
                    shapes.Add(new LineShape(lastCoordinate, destination));
                    lastCoordinate = destination;
                    i += 1;
                }
                else if (elements[i] == "V")
                {
                    Vector2 destination;
                    destination.X = lastCoordinate.X;
                    destination.Y = float.Parse(elements[i + 1], ci);
                    shapes.Add(new LineShape(lastCoordinate, destination));
                    lastCoordinate = destination;
                    i += 1;
                }
                else if (elements[i] == "z")
                {
                    shapes.Add(new LineShape(lastCoordinate, firstCoordinate));
                    i += 1;
                }
                else
                    i++; //TODO? - Throw exception instead?
            }
            return shapes;
        }
        private static List<Vector2> CreateSpawnPoint(float x, float y, float height, float width)
        {
            throw new NotImplementedException("Level.CreateSpawnPoint is not yet implemented");
        }

		public List<Entity> Entities
		{
			get
			{
				return this.entities;
			}
            private set
            {
                this.entities = value;
            }
		}
    }
}