﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
	class LineShape : IShape
	{
		public Vector2 Origin { get; set; }
		public Vector2 Destination { get; set; }
        public Sprite Sprite { get; set; }
		
        public LineShape(Vector2 origin, Vector2 destination)
		{
			Origin = origin;
			Destination = destination;
		}

        public void recalculateSize()
        {
            Sprite.SetSize((int)(Destination - Origin).Length() + 1, Sprite.Texture.Height / 2);
        }

        public Vector2 direction()
		{
			return (Destination - Origin);
		}

		public float length()
		{
			return (Destination - Origin).Length();
		}

		public float radian()
		{
			return (float)(Math.Atan2(Destination.Y - Origin.Y, Destination.X - Origin.X));
		}

		public Vector2 Center
		{
			get
			{
				return Origin + (Destination - Origin) / 2.0f;
			}
		}

        public void texturize()
        {
            texturize(ResourceManager.Instance.GetTexture("default_line"));
        }

        public void texturize(Texture2D texture)
        {
            Sprite = new Sprite(texture);
            Sprite.UseGlobalScaling = true;
            Sprite.Origin = new Vector2(1, texture.Height / 2);
            recalculateSize();
        }

        public void draw(SpriteBatch spriteBatch)
        {
            Sprite.Draw(spriteBatch, Origin, radian());
        }

        public Vector2[] getBoundingVolume()
        {
            return new Vector2[]
            {
                Origin,
                Destination
            };
        }

        public RayIntersection intersects(Ray ray)
        {
            RayIntersection intersection = null;

            Vector2 lineVector = Destination - Origin;
            Vector2 lineVectorPerp = new Vector2(lineVector.Y, -lineVector.X);
            Vector2 normal = new Vector2(lineVectorPerp.X, lineVectorPerp.Y);
            normal.Normalize();

            if (Vector2.Dot(normal, ray.Direction) != 0)
            {
                // Get the t value of ray. (t > 0)
                float t = Vector2.Dot(lineVectorPerp, Origin - ray.Origin)
                    / Vector2.Dot(lineVectorPerp, ray.Direction);

                // Get the s value of line segment. (0 < s < 1)
                float s = Vector2.Dot(new Vector2(ray.Direction.Y, -ray.Direction.X), Origin - ray.Origin)
                    / Vector2.Dot(lineVectorPerp, ray.Direction);

                if (t > 0 && s >= 0 && s <= 1) //Intersektion!!
                {
                    intersection = new RayIntersection();
                    intersection.Normal = normal;
                    intersection.T = t;
                }
            }

            return intersection;
        }
	}
}
