﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace MirrorMatch
{
	class Mirror: Entity
	{
        private IShape shape;

		public Mirror(IShape shape)
		{
            this.shape = shape;
            shape.texturize(ResourceManager.Instance.GetTexture("mirror"));
		}

        override public void Draw(SpriteBatch spriteBatch)
		{
            shape.draw(spriteBatch);
		}

		public override RayIntersection intersects(Ray ray)
		{
            RayIntersection intersection = shape.intersects(ray);
            if(intersection != null)
                intersection.IntersectingEntity = this;
            return intersection;
		}

        public override void onIntersection(LaserRay ray, RayIntersection intersection)
		{
            Vector2 projectionNormal = intersection.Normal * Vector2.Dot(ray.Direction, intersection.Normal);
            Vector2 reflectedDirection = ray.Direction - 2 * projectionNormal;
            //move the reflected ray a little bit away from the colliding surface
            Vector2 startPoint = ray.EndPoint + 0.5f*intersection.Normal;

            ray.createChild(startPoint, reflectedDirection, 0.95f); //create reflection
		}

        public override Vector2[] getBoundingVolume()
        {
            return shape.getBoundingVolume();
        }
	}
}
