﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Audio;

namespace MirrorMatch
{
    class ResourceManager
    {
        public static ResourceManager Instance { get; private set; }
        
        public static void Initialize(ContentManager content)
        {
            Instance = new ResourceManager(content);
        }


        private ContentManager content;
		private Dictionary<string, Texture2D> textures;
		private Dictionary<string, SpriteFont> fonts;
		private Dictionary<string, Level> levels;

        private ResourceManager(ContentManager content)
        {
            this.content = content;
            textures = new Dictionary<string, Texture2D>();
			fonts = new Dictionary<string, SpriteFont>();
			levels = new Dictionary<string, Level>();
        }

        public Texture2D GetTexture(string id)
        {
            return textures[id];
        }

        public void LoadTexture(string id, string fileName)
        {
            textures.Add(id, content.Load<Texture2D>(fileName));
        }

        public void LoadTextureFromFile(string id, string filePath)
        {
            //needed if we want to load levels dynamically without recompile
            //TODO: implement
        }

		public SpriteFont GetFont(string id)
		{
			return fonts[id];
		}

		public void LoadFont(string id, string fileName)
		{
			fonts.Add(id, content.Load<SpriteFont>(fileName));
		}

		public Level GetLevel(string id)
		{
			return this.levels[id];
		}

		public void LoadLevel(string id, string filename)
		{
			levels.Add(id, Level.LoadSvg(filename));
		}
    }
}
