﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
	class Sprite
	{
        public Texture2D Texture { get; private set; }
        public Vector2 Origin { get; set; }
        public Vector2 Scale { get; set; }
        public Color Tint { get; set; }
        public bool UseGlobalScaling = false;

		public Sprite(Texture2D textureImage = null)
		{
			Texture = textureImage;
            Scale = new Vector2(1, 1);
            Tint = Color.White;
		}

        /// <summary>
        /// Draw with this.Origin in global origo
        /// </summary>
        public void Draw(SpriteBatch spriteBatch)
        {
            Draw(spriteBatch, Vector2.Zero);
        }

		//Draw with this.Origin at given position
		public void Draw(SpriteBatch spriteBatch, Vector2 position)
		{
            Draw(spriteBatch, position, 0.0f);
		}

		/// <summary>
		/// Draw with this.Origin at given position, with given rotation around this.Origin
		/// </summary>
		public void Draw(SpriteBatch spriteBatch, Vector2 position, float rotation)
		{
            Draw(spriteBatch, position, rotation, new Vector2(0, 0));
		}

		/// <summary>
        /// Draw with this.Origin at given position, with given rotation around (this.Origin + relativeOrigin)
		/// </summary>
        public void Draw(SpriteBatch spriteBatch, Vector2 position, float rotation, Vector2 relativeOrigin)
		{
            if (UseGlobalScaling)
            {
                spriteBatch.Draw(Texture, position * MirrorMatch.ScalingFactor + relativeOrigin, Texture.Bounds,
                    Tint, rotation, Origin + relativeOrigin, Scale * MirrorMatch.ScalingFactor, SpriteEffects.None, 0);
            }
            else
            {
                spriteBatch.Draw(Texture, position + relativeOrigin, Texture.Bounds,
                   Tint, rotation, Origin + relativeOrigin, Scale, SpriteEffects.None, 0);
            }
		}

        public void SetSize(int width, int height)
        {
            Scale = new Vector2((float)width / Texture.Width, (float)height / Texture.Height);
        }
	}
}