﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
    class HUD
    {
        private Vector3 mCircleCenter; // A point that is to be transformed with the quad so as to stay with the latter's center.
        private Vector3 mRightPoint;
        private float outerHealthRadius;
        private float innerHealthRadius;
        private float outerEnergyRadius;
        private float innerEnergyRadius;

        private GraphicsDevice mGraphicsDevice;        
        private VertexBuffer mVertexBuffer;
        private IndexBuffer mIndexBuffer;
        private VertexPositionTexture[] vertexData;
        private int[] indices;
        private Effect mFX;

        private Matrix view;
        private Matrix proj;

        public void init(int windowWidth, int windowHeight, int size)
        {
            view = Matrix.CreateLookAt(new Vector3(windowWidth / 2, -windowHeight / 2, 3), new Vector3(windowWidth / 2, -windowHeight / 2, 0), new Vector3(0, 1, 0));
            proj = Matrix.CreateOrthographic(windowWidth, windowHeight, 0.1f, 100f);

            mGraphicsDevice = MirrorMatch.Instance.GraphicsDevice;
            mFX = MirrorMatch.Instance.Content.Load<Effect>("3D/hudFx");

            vertexData = new VertexPositionTexture[4];
            vertexData[0] = new VertexPositionTexture();
            vertexData[0].Position = new Vector3(-size, size, 0);
            vertexData[0].TextureCoordinate = new Vector2(0, 0);
            vertexData[1] = new VertexPositionTexture();
            vertexData[1].Position = new Vector3(size, size, 0);
            vertexData[1].TextureCoordinate = new Vector2(1, 0);
            vertexData[2] = new VertexPositionTexture();
            vertexData[2].Position = new Vector3(-size, -size, 0);
            vertexData[2].TextureCoordinate = new Vector2(0, 1);
            vertexData[3] = new VertexPositionTexture();
            vertexData[3].Position = new Vector3(size, -size, 0);
            vertexData[3].TextureCoordinate = new Vector2(1, 1);

            mVertexBuffer = new VertexBuffer(mGraphicsDevice,
                                             typeof(VertexPositionTexture), 4,
                                             BufferUsage.None);
            mVertexBuffer.SetData<VertexPositionTexture>(vertexData);

            indices = new int[6];
            indices[0] = 0;
            indices[1] = 1;
            indices[2] = 2;
            indices[3] = 2;
            indices[4] = 1;
            indices[5] = 3;

            mCircleCenter = new Vector3(0, 0, 0);
            mRightPoint = new Vector3(1, 0, 0);

            mFX.Parameters["View"].SetValue(view);
            mFX.Parameters["Proj"].SetValue(proj);
            mFX.Parameters["gCircleCenter"].SetValue(mCircleCenter);
            mFX.Parameters["gRightPoint"].SetValue(mCircleCenter);
        }

        public void draw(Vector2 position, float scalingFactor, float healthRatio, float energyRatio)
        {
            Matrix world = Matrix.CreateWorld(new Vector3(position.X, -position.Y, 0), new Vector3(0, 0, -1), new Vector3(0, 1, 0));
            world *= Matrix.CreateScale(scalingFactor);

            mFX.Parameters["World"].SetValue(world);
            mFX.Parameters["healthRatio"].SetValue(healthRatio);
            mFX.Parameters["energyRatio"].SetValue(energyRatio);

            mGraphicsDevice.SetVertexBuffer(mVertexBuffer);
            mFX.CurrentTechnique.Passes[0].Apply();
            mGraphicsDevice.DrawUserIndexedPrimitives(PrimitiveType.TriangleList, vertexData, 0, 4, indices, 0, 2);
        }
    }
}
