﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace MirrorMatch
{
    class LocalPlayer : Player
    {
        protected Ray[] intersectors;
        private LineShape aim;
		public Beam beam { get; set; }
        private InGameController controls;

        public LocalPlayer(Vector2 spawnPosition, string playerId, InGameController controller, ScoreData scoreData) : base(spawnPosition, playerId, scoreData)
        {
            this.controls = controller;

            intersectors = new Ray[]
            {
                new Ray(Position, new Vector2(0, -1)), //North
                new Ray(Position, new Vector2(0, 1)), //South
                new Ray(Position, new Vector2(-1, 0)), //West
                new Ray(Position, new Vector2(1, 0)), //East
                new Ray(Position,  new Vector2(-0.707106781f, -0.707106781f)), //North West
                new Ray(Position,  new Vector2(0.707106781f, -0.707106781f)), //North East
                new Ray(Position,  new Vector2(-0.707106781f, 0.707106781f)), //South West
                new Ray(Position,  new Vector2(0.707106781f, 0.707106781f)) //South East
            };

            aim = new LineShape(Position, Position + new Vector2(100,10));
        }

        public override void LoadContent()
        {
            base.LoadContent();

            //init a beam which will be recalculated every frame
            beam = new Beam(laserSprite, Id);

            aim.texturize(ResourceManager.Instance.GetTexture("lazor"));
            aim.Sprite.Tint = new Color(0.7f, 0.7f, 0.7f);
        }

        public override void Update(GameTime gameTime)
        {
            base.Update(gameTime);

            // Movement
            direction = controls.AimDirection(Position);
            direction.Normalize();

            speed = Vector2.Zero;

            if (controls.MoveLeft())
                this.speed.X -= 1.0f;
            if (controls.MoveRight())
                this.speed.X += 1.0f;
            if (controls.MoveUp())
                this.speed.Y -= 1.0f;
            if (controls.MoveDown())
                this.speed.Y += 1.0f;

            if (this.speed.Length() > 0)
            {
                UpdatePosition = true;
                this.speed.Normalize();
                this.speed *= 3f;
            }

            this.Position += speed;

            // Shoot laser? ---------------------------
            if (controls.Fire())
            {
                shooting = true;
                energy -= energyDrainSpeed;
                if (energy < 0)
                {
                    energy = 0;
                    shooting = false;
                }
            }
            else
            {
                shooting = false;
                energy += energyRechargeSpeed;
                if (energy > MAX_ENERGY)
                    energy = MAX_ENERGY;
            }// ----------------------------------------

            if (shooting)
            {
                KdTree.debugColoring = true; //start gathering debug data
                this.direction.Normalize();
                beam.recalculate(this.Position, this.direction, 100f);
                KdTree.debugColoring = false;//stop gathering debug data
            }
            else
            {
                //not shooting, update the aim, it will be shown in Draw()
                aim.Origin = Position;
                direction.Normalize();
                Ray aimRay = new Ray(Position, direction);
                RayIntersection hit = aimRay.findClosestIntersection();
                if (hit == null)
                {
                    aim.Destination = Position - direction * 100000;
                }
                else
                {
                    aim.Destination = Position - direction * hit.T;
                }
                aim.recalculateSize();
            }

            collideWithEntities();
        }

        public override void Draw(SpriteBatch spriteBatch)
        {
            if (shooting)
            {
                //draw the beam if currently shooting
                beam.draw(spriteBatch);
            }
            else
            {
                //draw the aim instead if not shooting
                aim.draw(spriteBatch);
            }
            base.Draw(spriteBatch);

            //GameObjectManager.GetInstance().StaticTree.drawDebugData(spriteBatch);
        }

        public override void onIntersection(LaserRay ray, RayIntersection intersection)
        {
            base.onIntersection(ray, intersection);

            //if(health <= 0)
            //    scoreData.RegisterVictory()
        }


        public void collideWithEntities()
        {
            //find intersection for each ray and calculate which intersection overlaps the player most
            RayIntersection closestHit = null;
            float largestOverlap = -1;

            foreach (Ray ray in intersectors)
            {
                //place rays on the edge of the circle, pointing inwards through the circle center
                ray.Origin = Position;

                RayIntersection hit = ray.findClosestIntersection();
                if (hit != null)
                {
                    //if it's an intersection, overlap will be between 0 to 2 * collisionRadius,
                    //since the rays are placed on the edge of the bounding circle, pointing inwards through the center
                    float overlap = collisionRadius - hit.T;

                    if (overlap > largestOverlap)
                    {
                        closestHit = hit;
                        largestOverlap = overlap;
                    }
                }
            }

            if (largestOverlap > 0.01)
            {
                //handle the intersection
                Position += closestHit.Normal * largestOverlap;
            }

        }
    }
}
