﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
//using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace MirrorMatch
{
	class Player: Entity
    {
        public const int MAX_HEALTH = 100;
        public const int MAX_ENERGY = 100;
        protected float energyRechargeSpeed = 1.0f;
        public float energyDrainSpeed = 1.5f;
        protected float healthDrain = 1.7f;

        public string Id { get; private set; }
        protected float health;
        protected float energy;
        public List<Vector2> offsetPoints { get; set; } //CHECK?
        protected Vector2 direction;
        protected Vector2 speed;
        protected List<TailPoint> tail;
        protected List<float> offsetLenght;
        protected Path pathLeft;
        protected Path pathRight;
        protected const int nrOfEdges = 50;
        protected const float tailLength = 70.0f;
        protected const float tailOffset = 10.0f;
        protected Sprite laserSprite;
        protected Sprite playerSprite;
        protected bool shooting;
        protected float collisionRadius;
        protected HUD hud;
		public ScoreData scoreData { get; set; }
        protected Vector2 spawnPoint;

		public Player(Vector2 spawnPosition, string playerId, ScoreData scoreData)
		{
            UpdatePosition = false;
            spawnPoint = spawnPosition;
            this.scoreData = scoreData;
            Id = playerId;
			this.Position = spawnPosition;
			direction = new Vector2(1f, 1f);
			direction.Normalize();
			pathLeft = new Path();
			pathRight = new Path();
            health = MAX_HEALTH;
            energy = MAX_ENERGY;

			collisionRadius = 13.0f;
			offsetLenght = new List<float>();
			offsetPoints = new List<Vector2>();

			tail = new List<TailPoint>();
			tail.Add(new TailPoint(this.Position, null));
			tail.Add(new TailPoint(this.Position - this.direction * tailLength / (float)nrOfEdges, tail.ElementAt(0)));
			for(int i = 2; i < nrOfEdges; i++)
			{
				// Tail generation
				tail.Add(new TailPoint(tail.ElementAt(i - 1).Position - this.direction * tailLength / (float)nrOfEdges, tail.ElementAt(i - 1)));

				// Offset generation
				float offset = tailOffset / i;
				offsetLenght.Add(offset);
			}
		}

		public virtual void LoadContent()
		{
			laserSprite = new Sprite(ResourceManager.Instance.GetTexture("lazor"));
			int offset = laserSprite.Texture.Height / 2;
			laserSprite.Origin = new Vector2(0, offset);
            laserSprite.UseGlobalScaling = true;
            laserSprite.Tint = Color.OrangeRed;

            
            Texture2D playerTexture = ResourceManager.Instance.GetTexture("mouse");
            playerSprite = new Sprite(playerTexture);
            playerSprite.Scale = new Vector2(0.125f, 0.125f);
            playerSprite.Origin = new Vector2(playerTexture.Width / 2, playerTexture.Height / 2);
            playerSprite.UseGlobalScaling = true;


            hud = new HUD();
            hud.init(MirrorMatch.Instance.WindowWidth, MirrorMatch.Instance.WindowHeight, 40);
		}

		public virtual void Update(GameTime gameTime)
		{
			#region Tail
			// Tail
			tail.ElementAt(0).Position = this.Position;
			tail.ElementAt(1).Position = this.Position - this.direction * tailLength / (float)nrOfEdges;

			offsetPoints.Clear();

			for (int i = 2; i < tail.Count; i++)
			{
				// Tail update
				Vector2 parentDir = tail.ElementAt(i - 2).Position - tail.ElementAt(i - 1).Position;
				Vector2 dir = tail.ElementAt(i - 1).Position - tail.ElementAt(i).Position;

				dir.Normalize();
				tail.ElementAt(i).Position = tail.ElementAt(i - 1).Position - dir * tailLength / (float)nrOfEdges;

				// Offset update
				Vector2 offsetDir = tail.ElementAt(i - 1).ParentPosition - tail.ElementAt(i - 1).Position;
				offsetDir = new Vector2(offsetDir.Y, -offsetDir.X);
				offsetDir.Normalize();
				offsetDir = offsetDir * offsetLenght.ElementAt(i - 2);
				offsetPoints.Add(tail.ElementAt(i - 1).Position + offsetDir);
				offsetPoints.Add(tail.ElementAt(i - 1).Position - offsetDir);
			}

			// Bezier things
			//PathLeft:
			pathLeft.clear();
			for (int i = 0; i < nrOfEdges - 2; i++)
			{
				pathLeft.addPoint(this.offsetPoints.ElementAt(2 * i));
			}
			pathLeft.addPoint(tail.Last().Position);
			//PathRight
			pathRight.clear();
			for (int i = 0; i < nrOfEdges - 2; i++)
			{
				pathRight.addPoint(this.offsetPoints.ElementAt(2 * i + 1));
			}
			pathRight.addPoint(tail.Last().Position);
			#endregion
		}

		public override void Draw(SpriteBatch spriteBatch)
		{            

			// Player-draw
            playerSprite.Draw(spriteBatch, Position, Radian);

            // Tail-draw
            Texture2D pointTexture = ResourceManager.Instance.GetTexture("point");
            //foreach (Point2d point in this.Tail)
            //{
            //    spriteBatch.Draw(pointTexture,
            //                point.Position * InGameState.ScalingFactor,
            //                new Rectangle(0, 0, pointTexture.Width, pointTexture.Height),
            //                Color.White,
            //                0,
            //                new Vector2(pointTexture.Width / 2, pointTexture.Height / 2),
            //                new Vector2(1f, 1f) * InGameState.ScalingFactor,
            //                SpriteEffects.None,
            //                0);
            //}

			// Tail-draw (outlines)
            //foreach (LineShape line in this.tailLines)
            //{
            //    spriteBatch.Draw(pointTexture,
            //                line.Destination * InGameState.ScalingFactor,
            //                new Rectangle(0, 0, pointTexture.Width, pointTexture.Height),
            //                Color.White,
            //                line.radian(),
            //                new Vector2(0, 0),
            //                new Vector2(line.length() * InGameState.ScalingFactor, 2f),
            //                SpriteEffects.None,
            //                0);
            //}
			
			// Offset-points-draw
            foreach (Vector2 vec in this.offsetPoints)
            {
                spriteBatch.Draw(pointTexture,
						vec * MirrorMatch.ScalingFactor,
                        pointTexture.Bounds,
                        Color.White,
                        0,
                        new Vector2(pointTexture.Width / 2, pointTexture.Height / 2),
						new Vector2(2f, 2f) * MirrorMatch.ScalingFactor,
                        SpriteEffects.None,
                        0);
            }

            //HUD draw
			hud.draw(this.Position, MirrorMatch.ScalingFactor, health / MAX_HEALTH, energy / MAX_ENERGY);
		}


		public override RayIntersection intersects(Ray ray)
		{   //Algorithm according to Real-Time Rendering, page 740
            Vector2 L = Position - ray.Origin;
		    float s = Vector2.Dot(L, ray.Direction);
            float L_square = Vector2.Dot(L, L);
            float r_square = collisionRadius * collisionRadius;
            //IF ray pointing away from circle center AND ray starting outside of circle
            if (s < 0 && L_square > r_square)
                return null;

            float m_square = L_square - s * s; //closest distance between the ray line and the circle center

            if (m_square > r_square)
                return null; //the ray line passes outside of the circle

            float q = (float) Math.Sqrt(r_square - m_square);


            float T;
            if (L_square > r_square)
            {
                T = s - q; //ray origin outside of the circle
            }
            else
            {
                T = s + q;
                //return null here to disable collision when the ray starts inside the player
                return null;
            }
                
            RayIntersection intersection = new RayIntersection();
			intersection.T = T;
            intersection.Normal = ray.Origin + ray.Direction * T - Position;
            intersection.IntersectingEntity = this;
			return intersection;
		}

        public override void onIntersection(LaserRay ray, RayIntersection intersection)
        {
            // Reduce health
            float newHealth = health - healthDrain;
            IntersectionResponse(ray.OwnerID, newHealth);
        }

        public virtual void IntersectionResponse(string rayOwnerId, float newHealth)
        {
            health = newHealth;
			UpdateHealth = true;

            // Calculate score
            float score = 0;
            if (health < 0)
            {
                score = healthDrain - health; // reduce score with negative health
            }
            else
                score = healthDrain;

            // Register score
            if (rayOwnerId == Id) // self inflicted hit, reduce own score
                scoreData.RegisterScore(rayOwnerId, -score);
            else // hit by other player, give that player score
                scoreData.RegisterScore(rayOwnerId, score);

            // Check for and handle player death
            if(health < 0)
                PlayerDeath(rayOwnerId);
        }

        public virtual void PlayerDeath(string killerId)
        {
            health = 0;
            if (killerId != Id)
                scoreData.RegisterVictory(killerId);
            Respawn();
        }

        protected virtual void Respawn()
        {
            Position = spawnPoint;
            UpdatePosition = true;
            UpdateHealth = true;
            RecentlyDied = true;
            health = MAX_HEALTH;
            energy = MAX_ENERGY;
        }

        public override Vector2[] getBoundingVolume()
        {
            return new Vector2[]
            {
                new Vector2(Position.X - collisionRadius, Position.Y - collisionRadius),
                new Vector2(Position.X - collisionRadius, Position.Y + collisionRadius),
                new Vector2(Position.X + collisionRadius, Position.Y - collisionRadius),
                new Vector2(Position.X + collisionRadius, Position.Y + collisionRadius)
            };

        }

		public List<LineShape> TailLines
		{
			get
			{
				List<LineShape> list = new List<LineShape>(pathLeft.PointsToLines.Union(pathRight.PointsToLines));
				return list;
			}
		}

		public List<TailPoint> Tail
		{
			get
			{
				return this.tail;
			}
		}

		public Vector2 Direction
		{
			get
			{
				return this.direction;
			}
			set
			{
				this.direction = value;
			}
		}

		public float Radian
		{
			get
			{
				return (float)Math.Atan2(direction.Y, direction.X) - (float)Math.PI/2f;
			}
		}

        public bool UpdatePosition { get; set; }
		public bool UpdateHealth { get; set; }
        public bool RecentlyDied { get; set; }

		public bool Shooting
		{
			get
			{
				return this.shooting;
			}
		}

		public float Health
		{
			get
			{
				return this.health;
			}
			set
			{
				health = value;
			}
		}
	}
}
