﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace MirrorMatch
{
	// A Convex Hull which defines 4 lines that surrounds an object
	class ConvexHull2D4
	{
		private List<Vector2> v;
        private List<LineShape> lineList;
        private Vector2 center;

		public ConvexHull2D4(Vector2 v0, Vector2 v1, Vector2 v2, Vector2 v3)
		{
			v = new List<Vector2>();
			v.Add(v0);
			v.Add(v1);
			v.Add(v2);
			v.Add(v3);
            center = (v[0] + v[1] + v[2] + v[3]) / 4;
            v.Sort(new Comparer(center));
            lineList = new List<LineShape>()
            {
                new LineShape(v[0], v[1]),
                new LineShape(v[1], v[2]),
                new LineShape(v[2], v[3]),
                new LineShape(v[3], v[0])
            };
		}

		public List<LineShape> LineList
		{
			get
            {
				return lineList;
			}
		}


        class Comparer : IComparer<Vector2>
        {
            Vector2 m_center;

            public Comparer(Vector2 center)
            {
                m_center = center;
            }

            private static float radian(Vector2 v)
            {
                return (float)Math.Atan2(v.Y, v.X);
            }

            public int Compare(Vector2 a, Vector2 b)
            {
                float deltaAngle = radian(m_center - b) - radian(m_center - a);
                return Math.Sign(deltaAngle);
            }
        }
	}
}
