﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace MirrorMatch
{
	class LaserRay
	{
        //guaranteed to be normalized
        public Vector2 Direction
        {
            get
            {
                return ray.Direction;
            }
        }
        public Vector2 StartPoint
        {
            get
            {
                return ray.Origin;
            }
        }
        public Vector2 EndPoint
        {
            get
            {
                return ray.Origin + ray.Direction * Length;
            }
        }
        public float Length { get; private set; }
		public float Energy { get; private set; }
        public string OwnerID { get; private set; }
        private Ray ray;
        private Sprite sprite;
        private List<LaserRay> children = new List<LaserRay>();
        private int nrOfChildren = 0;

        //re-defines ray energy, origin and direction, and recalculates the intersection.
        //the maxDistanceCollidable is used for distance culling against the kD-tree. 
        //Ray will not intersect with entities in kD-tree nodes further away than that.
        public void recalculate(float energy, Vector2 startPoint, Vector2 direction, float maxDistanceCollidable = float.MaxValue)
        {
            Energy = energy;
            nrOfChildren = 0;

            ray.Origin = startPoint;
            ray.Direction = direction;
            Length = 100000;

            RayIntersection closestHit = ray.findClosestIntersection(maxDistanceCollidable);

            if (closestHit != null)
            {
                Length = closestHit.T;
                closestHit.IntersectingEntity.onIntersection(this, closestHit);
            }
        }

        public LaserRay(Sprite sprite, string ownerId="Unknown")
        {
            OwnerID = ownerId;
            this.sprite = sprite;
            ray = new Ray(Vector2.Zero, new Vector2(0, 1));
        }

        public void createChild(Vector2 startPosition, Vector2 direction, float energyFactor = 1.0f)
        {
            if (Energy >= 50.0f)
            {
                //add a new child if needed, otherwise, just re-use old child
                nrOfChildren++;
                if (children.Count <= nrOfChildren)
                {
                    children.Add(new LaserRay(sprite, OwnerID));
                }
                children[nrOfChildren - 1].recalculate(Energy * energyFactor, startPosition, direction);
            }
        }

		public void draw(SpriteBatch spriteBatch)
		{
            for (int i = 0; i < nrOfChildren; i++ )
            {
                children[i].draw(spriteBatch);
            }
            sprite.SetSize((int)(EndPoint - StartPoint).Length(), 11);
            sprite.Draw(spriteBatch, StartPoint, getRadian());
		}

		public float getLength()
		{
            return (float)Math.Sqrt(Math.Pow((EndPoint.X - StartPoint.X), 2f)
                            + Math.Pow((EndPoint.Y - StartPoint.Y), 2f));
		}

		private float getRadian()
		{
            return (float)Math.Atan2(EndPoint.Y - StartPoint.Y, EndPoint.X - StartPoint.X);
		}

		public void getPoints(List<Vector2> pointList)
		{
			pointList.Add(StartPoint);
			pointList.Add(EndPoint);
			for (int i = 0; i < nrOfChildren; ++i)
			{
				children[i].getPoints(pointList);
			}
		}
	}
}
