﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace MirrorMatch
{
    class Ray
    {
        private Vector2 direction;
        public Vector2 Origin { get; set; }
        public Vector2 Direction
        {
            get
            {
                return direction;
            }
            set
            {
                direction = value;
                direction.Normalize();
            }
        }

        //maxDistanceStatic is used for distance culling against the kD-tree. tree nodes further away than that will be culled.
        public RayIntersection findClosestIntersection(float maxDistanceStatic = float.MaxValue)
        {
            //find the closest static entity
            RayIntersection closestHit = GameObjectManager.GetInstance().StaticTree.getClosestIntersection(this, maxDistanceStatic);

            //if any dynamic entity collides closer, use that RayIntersection instead
            foreach (Entity entity in GameObjectManager.GetInstance().DynamicEntities)
            {
                RayIntersection check = entity.intersects(this);
                if (check != null)
                {
                    if (closestHit == null || check.T < closestHit.T)
                    {
                        closestHit = check;
                    }
                }
            }

            //if there was a colission
			if (closestHit != null)
			{
                //make sure the normal points outwards from the colliding surface
                if (Vector2.Dot(closestHit.Normal, Direction) > 0)
                {
                    closestHit.Normal = -closestHit.Normal;
                }

			}

            return closestHit;
        }

        public Ray(Vector2 startPosition, Vector2 direction)
        {
            Origin = startPosition;
            Direction = direction;
        }
    }
}
