using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace MirrorMatch
{
	/// <summary>
	/// This is the main type for your game
	/// </summary>
	public class MirrorMatch : Microsoft.Xna.Framework.Game
	{
		//Game Basics:
		GraphicsDeviceManager   graphics;
		SpriteBatch             spriteBatch;
		Rectangle               clientBounds = new Rectangle(0, 0, 700, 700);
        bool                    enableFullscreen = true;

        //Game specific
        StateManager stateManager;

        //Properties
        public static MirrorMatch Instance { get; private set; }
        public static float ScalingFactor { get; set; }

		public MirrorMatch()
		{
            Instance = this;
			graphics = new GraphicsDeviceManager(this);
			Content.RootDirectory = "Content";
		}

		/// <summary>
		/// Allows the game to perform any initialization it needs to before starting to run.
		/// This is where it can query for any required services and load any non-graphic
		/// related content.  Calling base.Initialize will enumerate through any components
		/// and initialize them as well.
		/// </summary>
        protected override void Initialize()
        {
            InitGraphicsMode(clientBounds.Width, clientBounds.Height, enableFullscreen);
			ResourceManager.Initialize(Content);

            base.Initialize();
        }

		/// <summary>
		/// LoadContent will be called once per game and is the place to load
		/// all of your content.
		/// </summary>
		protected override void LoadContent()
		{
			// Create a new SpriteBatch, which can be used to draw textures.
			spriteBatch = new SpriteBatch(GraphicsDevice);

            ResourceManager.Initialize(Content);

            //Textures
            ResourceManager.Instance.LoadTexture("default_line", "Images/default_line");
            ResourceManager.Instance.LoadTexture("mirror", "Images/mirror");
            ResourceManager.Instance.LoadTexture("wall", "Images/wall");
			ResourceManager.Instance.LoadTexture("mouse", "Images/Mouse");
			ResourceManager.Instance.LoadTexture("point", "Images/point");
			ResourceManager.Instance.LoadTexture("lazor", "Images/LazorBeam");
            ResourceManager.Instance.LoadTexture("cursor", "Images/cursor");

			//Fonts
			ResourceManager.Instance.LoadFont("arial", "Fonts/arial");

			//Levels
			ResourceManager.Instance.LoadLevel("CircleArena", "Content/Levels/Circle Arena (Parsable Plain).svg");

            //State
            stateManager = new StateManager(new MainMenuState());
            stateManager.LoadContent();
		}

		/// <summary>
		/// UnloadContent will be called once per game and is the place to unload
		/// all content.
		/// </summary>
		protected override void UnloadContent()
		{
			stateManager.Cleanup();
		}

		/// <summary>
		/// Allows the game to run logic such as updating the world,
		/// checking for collisions, gathering input, and playing audio.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Update(GameTime gameTime)
		{
			// Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();
            //if (Keyboard.GetState().IsKeyDown(Keys.Escape))
            //    this.Exit();

            stateManager.Update(gameTime);

			base.Update(gameTime);
		}

		/// <summary>
		/// This is called when the game should draw itself.
		/// </summary>
		/// <param name="gameTime">Provides a snapshot of timing values.</param>
		protected override void Draw(GameTime gameTime)
		{
			//GraphicsDevice.Clear(new Color(0.1f, 0.1f, 0.1f));
            GraphicsDevice.Clear(new Color(0.1f,0.1f,0.1f));
			spriteBatch.Begin(SpriteSortMode.Deferred, BlendState.AlphaBlend); //Deferred means the sprites are rendered in drawing-order.

            stateManager.Draw(gameTime, spriteBatch);
            
			spriteBatch.End();

			base.Draw(gameTime);
		}

        //Makes it possible to change resolution and to/from full-screen mode.
        private bool InitGraphicsMode(int width, int height, bool fullScreen)
        {
            // If we aren't using a full screen mode, the height and width of the window can
            // be set to anything equal to or smaller than the actual screen size.
            if (fullScreen == false)
            {
                if ((width > GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Width)
                    || (height > GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Height))
                {
                    clientBounds.Width = width = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Width * 3 / 4;
                    clientBounds.Height = height = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode.Height * 3 / 4;
                }
            }
            else //Fullscreen mode
            {
                DisplayMode activeMode = GraphicsAdapter.DefaultAdapter.CurrentDisplayMode;

                clientBounds.Width = width = activeMode.Width;
                clientBounds.Height = height = activeMode.Height;

            }

            graphics.PreferredBackBufferWidth = width;
            graphics.PreferredBackBufferHeight = height;
            graphics.IsFullScreen = fullScreen;
            graphics.ApplyChanges();
            return true;
        }

        public int WindowWidth
        {
            get
            {
                return clientBounds.Width;
            }
        }

        public int WindowHeight
        {
            get
            {
                return clientBounds.Height;
            }
        }
	}
}
