﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace MirrorMatch
{
    class InGameState : State
    {
        public static float ScalingFactor{get; private set;}

        Level level;
        Player localPlayer;
        Cursor cursor;
        ScoreData scoreData;

        public InGameState()
        {
			this.level = ResourceManager.Instance.GetLevel("CircleArena");
            scoreData = new ScoreData();
            GameObjectManager.GetInstance().initStaticTree(Vector2.Zero, level.LevelDimensions);
            GameObjectManager.GetInstance().setStaticEntities(level.Entities);
            localPlayer = new LocalPlayer(new Vector2(100f, 100f), "You", new HumanPcController(), scoreData);
            localPlayer.energyDrainSpeed = 0;

            cursor = new Cursor();

            // Calculate and set scaling factor
            float ratioX = MirrorMatch.Instance.WindowWidth / level.LevelDimensions.X;
            float ratioY = MirrorMatch.Instance.WindowHeight / level.LevelDimensions.Y;
            MirrorMatch.ScalingFactor = Math.Min(ratioX, ratioY);
        }
        public override void LoadContent()
        {
            cursor.LoadContent();

            localPlayer.LoadContent();
            GameObjectManager.GetInstance().DynamicEntities.Add(localPlayer);
        }
        public override void Update(GameTime gameTime)
        {
            if (Keyboard.GetState().IsKeyDown(Keys.Escape))
                NextState = new MainMenuState();

            localPlayer.Update(gameTime);
            level.Update(gameTime);

            #if DEBUG
                Console.WriteLine("SCORE BOARD:\n" + scoreData.ScoreBoardAsString);
                Console.WriteLine("VICTORY BOARD:\n" + scoreData.VictoryBoardAsString);
            #endif
        }
        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            localPlayer.Draw(spriteBatch);
            level.Draw(gameTime, spriteBatch);

            // Draw mouse pointer:
            cursor.Draw(gameTime, spriteBatch);
        }
        public override void Cleanup()  //TODO?
        {
            //throw new NotImplementedException();
        }
    }
}
