﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

using System.IO;
using System.Net;
using System.Net.Sockets;
using System.Xml;

namespace MirrorMatch
{
	class InMultiplayerGameState: State
	{
		TcpClient client;
		Connection connection;
		private Level level;
		private LocalPlayer localPlayer;
		private OnlinePlayer onlinePlayer;
        private Cursor cursor;
		private string moveUpdate, dirUpdate, rayUpdate, healthUpdateLocal, healthUpdateOnline, scoreUpdate, deathUpdate;
		private Sprite spriteForOnlineRays;
		private List<Vector2> recvPointList;
		private Vector2 prevMousePos;
        private ScoreData scoreData;
        private bool hasConnection = false;
		private Vector2 localSpawnPosition;
		private Vector2 onlineSpawnPosition;
        private double nextSendTime = 0;
        private ASCIIEncoding encoder = new ASCIIEncoding();

		public InMultiplayerGameState(TcpClient client, Connection connection, Level level, string localPlayerId)
		{
            cursor = new Cursor();
			this.prevMousePos = Vector2.Zero;
			this.recvPointList = new List<Vector2>();
			this.client = client;
			this.connection = connection;
			this.level = level;
			moveUpdate = dirUpdate = rayUpdate = healthUpdateLocal = healthUpdateOnline = deathUpdate = "";
			GameObjectManager.GetInstance().initStaticTree(Vector2.Zero, level.LevelDimensions);
			GameObjectManager.GetInstance().setStaticEntities(level.Entities);
			string onlinePlayerId;
			if (localPlayerId == "Master")
			{
				onlinePlayerId = "Slave";
				localSpawnPosition = new Vector2(50f, 50f);
				onlineSpawnPosition = new Vector2(950f, 950f);
			}
			else
			{
				onlinePlayerId = "Master";
				localSpawnPosition = new Vector2(950f, 950f);
				onlineSpawnPosition = new Vector2(50f, 50f);
			}
            scoreData = new ScoreData(localPlayerId, onlinePlayerId);

			localPlayer = new LocalPlayer(localSpawnPosition, localPlayerId, new HumanPcController(), scoreData);
			onlinePlayer = new OnlinePlayer(onlineSpawnPosition, onlinePlayerId, scoreData);
            
            scoreData.VictoryLimit = 3;
            
			// Calculate scaling factor
			float ratioX = MirrorMatch.Instance.WindowWidth / level.LevelDimensions.X;
			float ratioY = MirrorMatch.Instance.WindowHeight / level.LevelDimensions.Y;
			MirrorMatch.ScalingFactor = Math.Min(ratioX, ratioY);
		}

		public override void LoadContent()
		{
            cursor.LoadContent();

			spriteForOnlineRays = new Sprite(ResourceManager.Instance.GetTexture("lazor"));
            spriteForOnlineRays.UseGlobalScaling = true;
            spriteForOnlineRays.Tint = Color.Aqua;

			localPlayer.LoadContent();
            onlinePlayer.LoadContent();
            GameObjectManager.GetInstance().DynamicEntities.Add(localPlayer);
            GameObjectManager.GetInstance().DynamicEntities.Add(onlinePlayer);
		}

		public override void Update(GameTime gameTime)
		{

			if (Keyboard.GetState().IsKeyDown(Keys.Escape)) //Not too good to try this yet.
				NextState = new MainMenuState();

            hasConnection = true;
            if (!client.Connected)
            {
                hasConnection = false;
                return;
            }

            NetworkStream clientStream = null;
            try
            {
                clientStream = client.GetStream();
            }
            catch
            {
                hasConnection = false;
                return;
            }


			localPlayer.Update(gameTime);
			onlinePlayer.Update(gameTime);
			level.Update(gameTime);

			// Send data to other player
			#region Send data

			// Send position
			if (localPlayer.UpdatePosition)
			{
				moveUpdate = "<move pos_x='" + localPlayer.Position.X.ToString() + "' pos_y='" + localPlayer.Position.Y.ToString() + "'></move>";
				localPlayer.UpdatePosition = false;
			}
			// Send rays
			if (localPlayer.Shooting)
			{
				List<Vector2> pointList = localPlayer.beam.getLines();
				rayUpdate = "";
				for (int i = 0; i < pointList.Count; i += 2)
				{
					rayUpdate += "<ray start_x='" + pointList.ElementAt(i).X.ToString() + "' start_y='" + pointList.ElementAt(i).Y.ToString() + "' end_x='" + pointList.ElementAt(i + 1).X.ToString() + "' end_y='" + pointList.ElementAt(i + 1).Y.ToString() + "'></ray>";
				}
			}
			else
			{
				rayUpdate = "<raystop></raystop>";
			}
			// Send direction
			if (new Vector2((Mouse.GetState().X - prevMousePos.X), (Mouse.GetState().Y - prevMousePos.Y)).Length() > 0)
			{
				dirUpdate = "<dir x='" + localPlayer.Direction.X.ToString() + "' y='" + localPlayer.Direction.Y.ToString() + "'></dir>";
			}
			// Send local health
			if(localPlayer.UpdateHealth)
			{
				healthUpdateLocal = "<hitplayer player_id='" + localPlayer.Id + "' hp='" + localPlayer.Health + "'></hitplayer>";
				localPlayer.UpdateHealth = false;
			}
			// Send online health
			if (onlinePlayer.UpdateHealth)
			{
				healthUpdateOnline = "<hitplayer player_id='" + onlinePlayer.Id + "' hp='" + onlinePlayer.Health + "'></hitplayer>";
				onlinePlayer.UpdateHealth = false;
			}
            if (onlinePlayer.RecentlyDied)
            {
                deathUpdate = "<death player_id='" + onlinePlayer.Id + "'></death>";
                onlinePlayer.RecentlyDied = false;
            }
			// Send score each frame
			scoreUpdate = "<score amount='" + localPlayer.scoreData.ScoreBoard[localPlayer.Id].ToString() + "'></score>";

			prevMousePos = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);
			if (gameTime.TotalGameTime.TotalMilliseconds >= nextSendTime)
			{
                nextSendTime = gameTime.TotalGameTime.TotalMilliseconds + 100;
				string sendString = moveUpdate + rayUpdate + dirUpdate + 
                    healthUpdateLocal + healthUpdateOnline + scoreUpdate + deathUpdate + "\n";

                try //SEND
                {
                    byte[] dirBuffer = encoder.GetBytes(sendString);
                    clientStream.Write(dirBuffer, 0, dirBuffer.Length);
                    clientStream.Flush();
                }
                catch
                {
                    hasConnection = false;
                }

				moveUpdate = rayUpdate = dirUpdate = healthUpdateLocal = healthUpdateOnline = deathUpdate = "";
			}
			#endregion

			// Receive data from other player
			#region Recv data
            string message = connection.ReadMessage();
			if (message != null)
			{
				// Put all messages into one single string
				string xmlString = "<xml>" + message + "</xml>";
                
                // -- Print received data for debug --
                //System.Diagnostics.Debug.WriteLine(xmlString);

				using (XmlReader reader = XmlReader.Create(new StringReader(xmlString)))
				{
					bool clear = true;
					// Parse the xml string and display each of the nodes.
					while (reader.Read())
					{
						switch (reader.NodeType)
						{
							case XmlNodeType.Element:
								float x, y, end_x, end_y;
								x = y = end_x = end_y = 0;
								string color = "";
								string player_id = "";
								float hp = 0;

								string elemName = reader.Name;

                                // If element has attributes
								if (reader.HasAttributes)
								{
									if (elemName == "move")
									{
										for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
											if (reader.Name == "pos_x")
												x = Convert.ToSingle(reader.Value);
											else if (reader.Name == "pos_y")
												y = Convert.ToSingle(reader.Value);
										}
										//System.Diagnostics.Debug.WriteLine("Position = new Vector2(" + x.ToString() + ", " + y.ToString() + ");");
										onlinePlayer.Position = new Vector2(x, y);
									}
									else if (elemName == "ray")
									{
										if (clear)
										{
											recvPointList.Clear();
											clear = false;
										}
										for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
											if (reader.Name == "start_x")
												x = Convert.ToSingle(reader.Value);
											else if (reader.Name == "start_y")
												y = Convert.ToSingle(reader.Value);
											else if (reader.Name == "end_x")
												end_x = Convert.ToSingle(reader.Value);
											else if (reader.Name == "end_y")
												end_y = Convert.ToSingle(reader.Value);
											else if (reader.Name == "color")
												color = reader.Value;
										}
										recvPointList.Add(new Vector2(x, y));
										recvPointList.Add(new Vector2(end_x, end_y));
										//System.Diagnostics.Debug.WriteLine("Origin = new Vector2(" + x.ToString() + ", " + y.ToString() + ");\nDestination = new Vector2(" + end_x.ToString() + ", " + end_y.ToString() + ");\ncolorStr = '" + color + "';");
									}
									else if (elemName == "hitplayer")
									{
										for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
											if (reader.Name == "player_id")
												player_id = reader.Value;
											else if (reader.Name == "hp")
												hp = Convert.ToSingle(reader.Value);
										}
										//System.Diagnostics.Debug.WriteLine("Player('" + player_id + "').setHp(" + hp.ToString() + ");");
										if (player_id == localPlayer.Id)
										{
											localPlayer.Health = hp;
										}
										else
										{
											onlinePlayer.Health = hp;
										}
									}
									else if (elemName == "dir")
									{
										for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
											if (reader.Name == "x")
												x = Convert.ToSingle(reader.Value);
											else if (reader.Name == "y")
												y = Convert.ToSingle(reader.Value);
										}
										onlinePlayer.Direction = new Vector2(x, y);
									}
									else if (elemName == "score")
									{
										for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
											if (reader.Name == "amount")
												x = Convert.ToSingle(reader.Value);
										}
										localPlayer.scoreData.ScoreBoard[onlinePlayer.Id] = x;
									}
                                    else if (elemName == "death")
                                    {
                                        for (int i = 0; i < reader.AttributeCount; i++)
										{
											reader.MoveToAttribute(i);
                                            if (reader.Name == "player_id")
                                                player_id = reader.Value;
										}
                                        if (player_id == localPlayer.Id)
                                            localPlayer.PlayerDeath(onlinePlayer.Id);
                                    }
                                    else
                                    {
                                        System.Diagnostics.Debug.WriteLine("unhandled network event: " + elemName);
                                    }
								}
								else // No attributes
								{
									if (elemName == "raystop")
									{
										recvPointList.Clear();
									}
								}

								break;
							case XmlNodeType.Text:
								System.Diagnostics.Debug.WriteLine(reader.Name);
								break;
							case XmlNodeType.XmlDeclaration:
							case XmlNodeType.ProcessingInstruction:
								break;
							case XmlNodeType.Comment:
								break;
							case XmlNodeType.EndElement:
								break;
						}
					}
				}
			}
			#endregion


			// Next state

            if (scoreData.PlayerHasWon)
                NextState = new GameOverState(scoreData/*, client, connection, level, localPlayer.Id*/);

//#if DEBUG
//                Console.WriteLine("SCORE BOARD:\n" + scoreData.ScoreBoardAsString);
//                Console.WriteLine("VICTORY BOARD:\n" + scoreData.VictoryBoardAsString);
//#endif
		}

		public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
		{
			localPlayer.Draw(spriteBatch);
			onlinePlayer.Draw(spriteBatch);
			level.Draw(gameTime, spriteBatch);
			for(int i = 0; i < recvPointList.Count; i+=2)
			{
				Vector2 start = recvPointList[i];
				Vector2 end = recvPointList[i+1];
				spriteForOnlineRays.SetSize((int)(start - end).Length(), 11);
				spriteForOnlineRays.Origin = new Vector2(0, spriteForOnlineRays.Texture.Height/2);
				spriteForOnlineRays.Draw(spriteBatch, start, (float)Math.Atan2(end.Y - start.Y, end.X - start.X));
			}

            if (!hasConnection)
            {
                //spriteBatch.DrawString(ResourceManager.Instance.GetFont("arial"), "* NO CONNECTION *", new Vector2(100, 100), Color.Yellow);
                NextState = new GameOverState(scoreData);
            }
			spriteBatch.DrawString(ResourceManager.Instance.GetFont("arial"), localPlayer.scoreData.ScoreBoardAsString, new Vector2(400f, 20f), Color.CornflowerBlue);

			// Draw mouse pointer:
            cursor.Draw(gameTime, spriteBatch);
		}

		public override void  Cleanup()
        {
            if (connection != null)
                connection.Cleanup();
		}
	}
}
