﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace MirrorMatch
{
    class MainMenuState : State
    {
        //Texture2D background;
        Sprite cursorSprite;
        MenuButton startGameButton;
        MenuButton exitButton;
		MenuButton createGameButton;
		MenuButton joinGameButton;

        public MainMenuState()
        { 
            // General button settings
            SpriteFont buttonFont = ResourceManager.Instance.GetFont("arial");
            Color buttonColor = new Color(0, 130, 255);
            Color textColor = Color.White;

            // StartGame button
            string buttonText = "Ray Demo";
            Vector2 textSize = buttonFont.MeasureString(buttonText);
            Rectangle buttonBounds = new Rectangle(
                (int)(MirrorMatch.Instance.WindowWidth / 2 - 110 / 2),
                (int)(MirrorMatch.Instance.WindowHeight / 8 - 50 / 2),
                125, 50);
            startGameButton = new MenuButton(buttonText, buttonFont, textColor, buttonColor, buttonBounds);

            // Exit button
            buttonText = "Exit";
            textSize = buttonFont.MeasureString(buttonText);
            buttonBounds = new Rectangle(
                (int)(MirrorMatch.Instance.WindowWidth / 2 - 110/2),
                (int)(3.1f * MirrorMatch.Instance.WindowHeight / 8 - 50 / 2),
                125, 50);
            exitButton = new MenuButton(buttonText, buttonFont, textColor, buttonColor, buttonBounds);

			// Create Game button
			buttonText = "Create Game";
			textSize = buttonFont.MeasureString(buttonText);
			buttonBounds = new Rectangle(
				(int)(MirrorMatch.Instance.WindowWidth / 2 - 110 / 2),
				(int)(2.4f * MirrorMatch.Instance.WindowHeight / 8 - 50 / 2),
				125, 50);
			createGameButton = new MenuButton(buttonText, buttonFont, textColor, buttonColor, buttonBounds);
			
			// Join Game button
			buttonText = "Join Game";
			textSize = buttonFont.MeasureString(buttonText);
			buttonBounds = new Rectangle(
				(int)(MirrorMatch.Instance.WindowWidth / 2 - 110 / 2),
                (int)(1.7f * MirrorMatch.Instance.WindowHeight / 8 - 50 / 2),
				125, 50);
			joinGameButton = new MenuButton(buttonText, buttonFont, textColor, buttonColor, buttonBounds);
        }
        public override void LoadContent()
        {
            cursorSprite = new Sprite(ResourceManager.Instance.GetTexture("cursor"));
            cursorSprite.SetSize(12, 18);
            cursorSprite.UseGlobalScaling = false;
        }
        public override void Update(GameTime gameTime)
        {
            // StartGame button
            if (startGameButton.ButtonClicked)
                NextState = new InGameState();
            if (startGameButton.IsHoveredOver)
                startGameButton.TextColor = Color.Red;
            else
                startGameButton.TextColor = Color.White;

            // Exit button
            if (exitButton.ButtonClicked)
                MirrorMatch.Instance.Exit();
            if (exitButton.IsHoveredOver)
                exitButton.TextColor = Color.Red;
            else
                exitButton.TextColor = Color.White;

			//Temporary buttons ###################################################################################################
			// Create Game button
			if (createGameButton.ButtonClicked)
				NextState = new CreateGameState();
			if (createGameButton.IsHoveredOver)
				createGameButton.TextColor = Color.Red;
			else
				createGameButton.TextColor = Color.White;

			// Join Game button
			if (joinGameButton.ButtonClicked)
				NextState = new JoinGameState();
			if (joinGameButton.IsHoveredOver)
				joinGameButton.TextColor = Color.Red;
			else
				joinGameButton.TextColor = Color.White;
        }
        public override void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            // Draw buttons
            startGameButton.Draw(gameTime, spriteBatch);
            exitButton.Draw(gameTime, spriteBatch);

			//Temporary buttons ###################################################################################################
			createGameButton.Draw(gameTime, spriteBatch);
			joinGameButton.Draw(gameTime, spriteBatch);
            
            // Draw mouse pointer:
            Vector2 mousePos = new Vector2(Mouse.GetState().X, Mouse.GetState().Y);
            cursorSprite.Draw(spriteBatch, mousePos);
        }
        public override void Cleanup()  //TODO?
        {
            //throw new NotImplementedException();  
        }
    }
}
