﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace MirrorMatch
{
    class MenuButton
    {
        #region FIELDS & PROPERTIES WITH BACKING FIELDS
        SpriteFont  font;
        Vector2     textPosition;
        Texture2D   fillTexture;
        //Sprite    cornerSprite;
        //Sprite    borderSprite;

        Color fillColor;
        public Color FillColor
        {
            get { return fillColor; }
            set { fillColor = value; }
        }

        Color textColor;
        public Color TextColor
        {
            get { return textColor; }
            set { textColor = value; }
        }

        string buttonText;
        public string ButtonText
        {
            get { return buttonText; }
            set //TODO?
            {
                //Vector2 textSize = font.MeasureString(value);
                //if (textSize.X < buttonBounds.Width || textSize.Y < buttonBounds.Height)
                    buttonText = value;
                //else
                //    throw new ArgumentOutOfRangeException("Text \"" + value + "\" too long. Make button bigger or text shorter");
            }
        }

        Rectangle buttonBounds;
        public Rectangle ButtonBounds
        {
            get { return buttonBounds; }
            set 
            { 
                buttonBounds = value;
                //  bounds changed, center text
                Point center = buttonBounds.Center;
                Vector2 textSize = font.MeasureString(buttonText);
                textPosition = new Vector2(center.X - textSize.X / 2, center.Y - textSize.Y / 2);
            }
        }
        #endregion

        #region READ ONLY & CALCULATED PROPERTIES
        public Vector2 Position
        {
            get { return new Vector2(buttonBounds.X, buttonBounds.Y); }
            set
            {
                Rectangle newBounds = new Rectangle((int)value.X, (int)value.Y, buttonBounds.Width, buttonBounds.Height);
                ButtonBounds = newBounds;
            }
        }
        public bool ButtonClicked
        {
            get
            {   
                MouseState mouse = Mouse.GetState();
                if (buttonBounds.Contains(mouse.X, mouse.Y)
                    && mouse.LeftButton == ButtonState.Pressed)
                    return true;
                else
                    return false;
            }
        }
        public bool IsHoveredOver
        {
            get
            {
                MouseState mouse = Mouse.GetState();
                if (buttonBounds.Contains(mouse.X, mouse.Y))
                    return true;
                else
                    return false;
            }
        }
        #endregion

        #region INSTANCE METHODS
        public void Draw(GameTime gameTime, SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(fillTexture, buttonBounds, fillColor);
            
            spriteBatch.DrawString(
                font, 
                buttonText,
                textPosition,
                textColor);

        }
        #endregion

        #region CONSTRUCTORS
        public MenuButton(string buttonText, SpriteFont font, Color textColor, Color fillColor, Rectangle buttonBounds)
        {
            this.font = font;
            this.fillColor = fillColor;
            this.textColor = textColor;
            ButtonText = buttonText;
            ButtonBounds = buttonBounds;
            fillTexture = new Texture2D(MirrorMatch.Instance.GraphicsDevice, 1, 1);
            fillTexture.SetData<Color>( new Color[] { Color.White });
        }
        #endregion
    }
}
