//Input
float4x4 World;
float4x4 View;
float4x4 Proj;
float healthRatio = 1;
float energyRatio = 1;

// TODO: add effect parameters here.
float3 gCircleCenter;
float3 gRightPoint;

struct VertexShaderInput
{
    float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Proj);

	output.TexCoord = input.TexCoord;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{

	float outerHealthRadius = 0.30f;
	float innerHealthRadius = 0.25f;

	float outerEnergyRadius = 0.40f;
	float innerEnergyRadius = 0.35f;

	float2 texCoord = input.TexCoord - (0.5,0.5);

	float2 vec = texCoord - gCircleCenter.xy;

	float ratio = ((atan2(texCoord.y, texCoord.x) + 2.5*3.1416)%(2*3.1416)) / (2*3.1416);
	
	float dist = sqrt(vec.x * vec.x + vec.y * vec.y);

	if( (dist <= outerHealthRadius + 0.0125) && 
	    (dist >= outerHealthRadius)
			||
		(dist <= innerHealthRadius + 0.0125) && 
	    (dist >= innerHealthRadius)
			||
		(dist <= outerEnergyRadius + 0.0125) && 
	    (dist >= outerEnergyRadius)
			||
		(dist <= innerEnergyRadius + 0.0125) && 
	    (dist >= innerEnergyRadius))
	{
		//no border
		return float4(0, 0, 0, 0);
	}
	else if( (dist <= outerHealthRadius) && 
	         (dist >= innerHealthRadius) &&
			 (ratio <= healthRatio) )
	{
		return float4(1, 0, 0, 1);
	}
	else if( (dist <= outerEnergyRadius) && 
	         (dist >= innerEnergyRadius) &&
			 (ratio <= energyRatio) )
	{
		return float4(0, 0.7f, 1, 1);
	}
	

    return float4(0, 0, 0, 0);
}

technique Technique1
{
    pass Pass1
    {
        VertexShader = compile vs_3_0 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
